# Request API

Request  messages are sent when the plugin requests siibra-explorer to do something on its behalf.

Be it request the user to select a region, a point, navigate to a specific location etc. 

> :warning: Please note that `beforeunload` window event does not fire on iframe windows. Plugins should do whatever cleanup it needs, then send the message `sxplr.exit`. 

```javascript

let parentWindow
window.addEventListener('message', ev => {
  const { source, data, origin } = msg
  const { id, method, params, result, error } = data

  if (method === "sxplr.init") {
    parentWindow = source
  }
})

window.addEventListener('pagehide', () => {

  // do cleanup
  // n.b. since iframe unload usually do not trigger DOM events
  // one will need to manually trigger destroying any apps manually

  parentWindow.postMessage({
    jsonrpc: '2.0',
    method: `sxplr.exit`,
    params: {
      requests: [] // any remaining requests to be carried out
    }
  })
})
```

<!-- the API reference below are auto generated by generateTypes.js  -->
<!-- do not edit, as the edit will be overwritten by the auto generation -->

## API
### `sxplr.getAllAtlases`

- request

  ```ts
  null
  ```

- response

  ```ts
  SapiAtlasModel[]
  ```


### `sxplr.getSupportedTemplates`

- request

  ```ts
  null
  ```

- response

  ```ts
  SapiSpaceModel[]
  ```


### `sxplr.getSupportedParcellations`

- request

  ```ts
  null
  ```

- response

  ```ts
  SapiParcellationModel[]
  ```


### `sxplr.selectAtlas`

- request

  ```ts
  {"@id": string}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.selectParcellation`

- request

  ```ts
  {"@id": string}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.selectTemplate`

- request

  ```ts
  {"@id": string}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.navigateTo`

- request

  ```ts
  MainState['[state.atlasSelection]']['navigation'] & {"animate": boolean}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.getUserToSelectARoi`

- request

  ```ts
  {"type": 'region' | 'point', "message": string}
  ```

- response

  ```ts
  SapiRegionModel | OpenMINDSCoordinatePoint
  ```


### `sxplr.addAnnotations`

- request

  ```ts
  {"annotations": SxplrCoordinatePointExtension[]}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.rmAnnotations`

- request

  ```ts
  {"annotations": AtId[]}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.loadLayers`

- request

  ```ts
  {"layers": AddableLayer[]}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.updateLayers`

- request

  ```ts
  {"layers": AddableLayer[]}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.removeLayers`

- request

  ```ts
  {"layers": {"id": string}}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.exit`

- request

  ```ts
  {"requests": JRPCRequest[]}
  ```

- response

  ```ts
  'OK'
  ```


### `sxplr.cancelRequest`

- request

  ```ts
  {"id": string}
  ```

- response

  ```ts
  'OK'
  ```

